#include <QtGui>
#include <QtNetwork/QtNetwork>
#include "hanoi.h"



/*!
    \brief  Constructor of the class, initialize game and update graphics
*/
Hanoi::Hanoi(QWidget *parent)
    : QMainWindow(parent)

{
    // Set the game size
    setFixedSize(Xw+40, Yw+40);
    scene =  new QGraphicsScene;

    // Deflaut initilization (7 pieces)
    Init (7);


    vue = new QGraphicsView(scene, this);
    vue->setRenderHints(QPainter::Antialiasing|QPainter::TextAntialiasing);
    vue->setGeometry(10, 10, Xw+20, Yw+20);
    vue->setStyleSheet("background-color: #FFFFFF");


    udpSocket = new QUdpSocket(this);
    udpSocket->bind(45454, QUdpSocket::ShareAddress);
    connect(udpSocket, SIGNAL(readyRead()),this, SLOT(processPendingDatagrams()));

    DrawGame();
}


/*!
     \brief     process incomming sockets
*/
void Hanoi::processPendingDatagrams()
{
    int From,To;
    while (udpSocket->hasPendingDatagrams()) {
        char Data[10];
        udpSocket->readDatagram(Data, 10);

        if (Data[0]=='R' and Data[1]=='E')
        {
            int n;
            sscanf(&Data[2],"%d",&n);
            if (n<0) n=1;
            if (n>MaxPieces) n=MaxPieces;
            Init(n);
            DrawGame();
            qDebug () << Data << "Restart the game with" << n << "Disk(s)";
            return;
        }

        if (Data[0]=='M' and Data[1]=='V')
        {
            switch (Data[2])
            {
            case '0': From=0; break;
            case '1': From=1; break;
            case '2': From=2; break;
            default:return;
            }
            switch (Data[3])
            {
            case '0': To=0; break;
            case '1': To=1; break;
            case '2': To=2; break;
            default:return;
            }
            if (MovePart(From,To))
                qDebug () << Data << "Move a disk from" << To << "to" << From;
            else
                qDebug() << "Error while moving";
        }
    }
}

/*!
    \brief  Desctructor, delete objects
*/
Hanoi::~Hanoi()
{
    delete (scene);
    delete (udpSocket);
}


/*!
    \brief Initialize game and graphics
*/
void Hanoi::Init(int NbPieces)
{
    int i;
    scene->clear();
    scene->addRect(-Xw/2,0,Xw,10,QPen(Qt::black),QBrush(Qt::gray));
    scene->addRect(-5,0,10,-Yw+10,QPen(Qt::black),QBrush(Qt::gray));
    scene->addRect(-5-Xw/3,0,10,-Yw+10,QPen(Qt::black),QBrush(Qt::gray));
    scene->addRect(-5+Xw/3,0,10,-Yw+10,QPen(Qt::black),QBrush(Qt::gray));

    if (NbPieces>MaxPieces) return;
    PieceHeight=(Yw-20)/NbPieces;

    for (i=0;i<NbPieces;i++) {
        Part[i].width= ((Xw/3.0)-10.0)-i*(((Xw/3.0)-10.0)-30.0)/NbPieces ;
        Part[i].height=PieceHeight;
        Part[i].Drawing=scene->addRect(0,
                                       0,
                                       Part[i].width,
                                       -Part[i].height,
                                       QPen(Qt::black),
                                       QBrush (Qt::blue));
        Game[0][i]=i;
    }
    Game[0][i]=-1;
    Game[1][0]=-1;
    Game[2][0]=-1;

}


/*!
    \brief Update display of the game
*/
void Hanoi::DrawGame()
{
    for (int i=0;i<3;i++)
        for (int j=0; Game[i][j]!=-1;j++)
        {
        int NumP=Game[i][j];
        QGraphicsItem *PtObj=Part[NumP].Drawing;
        PtObj->resetTransform();
        PtObj->setTransform(QTransform::fromTranslate(-Part[NumP].width/2 + (i-1)*(Xw/3),-j*PieceHeight),true);
    }
    vue->update(0,0,Xw+20,Yw+20);
}



/*!
    \brief  Move a part from From to To if it is possible
*/
int Hanoi::ChangePart(int From, int To)
{
    int i,j;
    for (i=0;Game[From][i]!=-1;i++);
    for (j=0;Game[To][j]!=-1;j++);
    if (i==0) return 0;
    if (j!=0) {
        if (Game[To][j-1]>Game [From][i-1])
            return 0;
    }
    Game [To][j]=Game [From][i-1];
    Game [From][i-1]=-1;
    Game [To][j+1]=-1;
    return 1;
}



/*!
    \brief  Move a part from From to To and update display
*/
int Hanoi::MovePart(int From, int To)
{
    int res=ChangePart(From,To);
    DrawGame();
    return res;
}
